#if !defined _SelectorList_h_
#define _SelectorList_h_

// helper class to provide safer critical sections
class CritSec
{
    public:
        CritSec()
        {
            InitializeCriticalSection( &m_CritSec );
        }

        ~CritSec()
        {
            DeleteCriticalSection( &m_CritSec );
        }

        class Lock
        {
            public:
                Lock( CritSec& critSec ) : m_pCritSec( &critSec )
                {
                    EnterCriticalSection( &m_pCritSec->m_CritSec);
                }

                ~Lock()
                {
                    LeaveCriticalSection( &m_pCritSec->m_CritSec );
                    m_pCritSec = 0;
                }

            private:
                CritSec* m_pCritSec;
        };

    private:
        friend class Lock;
        CRITICAL_SECTION m_CritSec;
};

class SelectorList
{
    public:
    // LIFECYCLE
    //
        SelectorList( HWND hwndParent,
                      const char* szTitle,
                      int configFlags,
                      int positionFlags,
                      const RECT& posRect );

        ~SelectorList();
        bool Create();

    // OPERATIONS
    //
        void SetRedraw( bool enabled );
        void ClearContent();
        void InsertString( const char* szData );
        void SetTitle( const char* szTitle );
        void SelectString( const char* szStr );
        void ResetChoice();
        void OnParentMove();
        bool Choice( int& idx, bool& forRemoval, bool reset );
        void ExitTSE( bool flag ) { m_fClosed = flag; }

    // ACCESS
    //
        string Title() const { return m_title; }

    // INQUIRY
    //
        // configurable options
        bool ShrinkOnClick() const { return Flags() & e_fShrinkOnClick; }
        bool SingleClickActivate() const { return Flags() & e_fSingleClick; }
        bool Shrunk() const { return m_fShrunk; }
        bool AllowClose() const { return !(Flags() & e_fNoClose); }
        bool ShowNumFiles() const { return Flags() & e_fShowNumFiles; }
        bool IsAutoSizing() const { return Flags() & e_fAutoSize; }
        bool AlwaysOnTop() const { return Flags() & e_fAlwaysOnTop; }
        bool SmallFont() const { return Flags() & e_fSmallFont; }
        bool NoSizing() const { return Flags() & e_fNoSize; }
        bool NoDragging() const { return Flags() & e_fNoDrag; }
        bool ChangeIcon() const { return Flags() & e_fChangeIcon; }
        bool ScreenSaver() const { return Flags() & e_fMidBtnScrSvr; }
        bool AlternateIcon() const { return Flags() &e_fAlternateIcon; }
        bool UseTaskBar() const { return Flags() & e_fTaskBar; }
        bool CloseTSE() const { return Flags() & e_fCloseTse; }
        bool ShrinkTSE() const { return Flags() & e_fShrinkTse; }

        bool ParentMoved() const;
        bool ExitTSE() const { return m_fClosed; }

    protected:
    // OPERATIONS
    //
        void OnChar( char ch );
        void OnLButtonDown( WPARAM wParam );
        void OnLButtonDblClick( WPARAM wParam );
        void OnRButtonDown( WPARAM wParam );
        bool OnDragging();
        void ResetSearch() { m_searchStr = ""; }
        void ShrinkExpand();

    private:
    // OPERATIONS
    //
        void ItemSelected( bool forRemoval );
        void AutoSize();
        void CalcDimensions( RECT& rect );
        void UpdatePosition( const RECT& rect );
        void SetChoice( int idx, bool forRemoval );
        static LRESULT FAR PASCAL CallBack( HWND hwnd,
                                            UINT message,
                                            WPARAM wParam,
                                            LPARAM lParam );
        LRESULT OnMessage( UINT message,
                           WPARAM wParam,
                           LPARAM lParam );

    // ACCESS
    //
        HWND Hwnd() const { return m_hwnd; }
        HWND Parent() const { return m_hwndParent; }
        int Flags() const { return m_CfgFlags; }
        int PosFlags() const { return m_PositionFlags; }

    // INQUIRY
    //
        bool IsParentMinimized() const;


    // DATA MEMBERS
    //
        HWND m_hwnd;
        HWND m_hwndParent;
        string m_searchStr;

        bool m_fShrunk;
        bool m_fHidden;

        int m_iChoice;
        bool m_fRemoveFile;

        bool m_fDragging;

        RECT m_fullSizeRect;
        RECT m_parentBigRect;

        int m_CfgFlags;
        int m_PositionFlags;

        string m_title;
        bool m_fClosed;

        RECT m_rectParent;      // save parent pos to determine if changed
        WINDOWPLACEMENT m_wndPlacement;  // placement of parent when minimized

        HFONT m_hFont;
        CritSec m_CritSec;

        static FARPROC m_pListBoxHandler;

    // ENUMERATIONS
    //
        // corresponds to flags in select.s
        enum ConfigFlags
        {
            e_fAlwaysOnTop     = 0x0001, // force to be topmost window
            e_fSingleClick     = 0x0002, // single click selects file
            e_fNoClose         = 0x0004, // don't allow window to close
            e_fShrinkOnClick   = 0x0008, // shrink on right click
            e_fShowNumFiles    = 0x0010, // show number of files in caption
            e_fAutoSize        = 0x0020, // auto adjust height to content
            e_fSmallFont       = 0x0040, // use smaller font
            e_fNoSize          = 0x0080, // don't allow window to be sized
            e_fNoDrag          = 0x0100, // don't allow window to be dragged
            e_fMidBtnScrSvr    = 0x0200, // middle button activates screen saver
            e_fChangeIcon      = 0x0400, // change TSE icon to windows logo
            e_fAlternateIcon   = 0x0800, // change TSE icon
            e_fTaskBar         = 0x1000, // place icon in system tray
            e_fCloseTse        = 0x2000, // closing selector also closes TSE
            e_fShrinkTse       = 0x4000  // shrinking selector also shrinks TSE
        };

        enum PositionFlags
        {
            e_fPosBindLeft     = 0x0001,
            e_fPosBindRight    = 0x0002,
            e_fPosAbsolute     = 0x0004,

            // modifier - treat all dimensions as percentage of screen width
            e_fAsPercent       = 0x1000
        };
};

#endif      // !defined _SelectorList_h_
