//////////////////////////////////////////////////////////////////////////////
// SELECT.CPP           Chris Lee       October 16, 1997
//
// Selector v1.12 - TSE/32 GUI file selector
//
// Starts a GUI thread to show open files in TSE.  User selected file is then
// made current in TSE.
//
//
// Development Platform:
//
//      Borland C++ 5.01
//      Windows NT 4.0 SP3
//      Rogue Wave Standard C++ Library
//
// Notes:
//     -See SELECT.S for proper usage of interface
//     -Recompilation should be possible with a conforming Standard C++ library
//      and a decent compiler (Borland, MSVC).  The debugging stuff is custom
//      and can be commented out.  Change _beginthreadNT if you change compilers
//     -no locking of data structures is done as the message queue provides
//      synchronization
//
//
//////////////////////////////////////////////////////////////////////////////

#include <windows.h>
#include <debug.h>
#include <process.h>
#include <string.h>
#include <vector.h>
#include <set.h>

#pragma hdrstop

#include "fileinfo.h"
#include "listbox.h"
#include "system.h"
#include "globals.h"

// debug structure (expanded if __DEBUG=2 and __TRACE are defined )
DBG_DEFINE_GROUP( Def, 1, 9 );

//////////////////////////////////////////////////////////////////////////////
// TYPEDEFS
//////////////////////////////////////////////////////////////////////////////
typedef vector< FileInfo*, allocator > FileInfoVec;
typedef set< FileInfo, less< FileInfo >, allocator > FileInfoSet;

FileInfoVec g_vecFileInfoPtr;   // vector of pointers to FileInfo objects
FileInfoSet g_setFileInfo;      // set of FileInfo objects



//////////////////////////////////////////////////////////////////////////////
// EXPORTS
//////////////////////////////////////////////////////////////////////////////
extern "C"
{
bool __export __stdcall Select_Init();
int __export __stdcall Select_AddFile( const char* fileName,
                                       int bufferId,
                                       bool fFileChanged );
int __export __stdcall Select_RemoveFile( const char* fileName, int bufferId );
int __export __stdcall Select_OnIdle();
void __export __stdcall Select_Redraw();
void __export __stdcall Select_HiliteCurrentFile( const char* fileName );

bool __export __stdcall Select_Config( HWND hwndParent,
                                       const char* szTitle,
                                       int configFlags,
                                       int positionFlags,
                                       int left,
                                       int top,
                                       int width,
                                       int height,
                                       int instances );
int __export __stdcall Select_ExitTSE();
};

bool __export __stdcall Select_Config( HWND hwndParent,
                                       const char* szTitle,
                                       int configFlags,
                                       int positionFlags,
                                       int left,
                                       int top,
                                       int width,
                                       int height,
                                       int instances )
{
    if( instances < 0 )
    {
        TRACE( "Invalid instances: " << instances );
        return false;
    }

    if( instances > 0 )
    {
        // create semaphore (or open existing semaphore of same name)
        g_hSem = CreateSemaphore( 0, instances, instances, "TSE32 Selector" );
        if( 0 == g_hSem )
        {
            TRACE( "Error creating semaphore: " << GetLastError() );
            return false;
        }

        DWORD dwResult = WaitForSingleObject( g_hSem, 0 );

        if( dwResult == WAIT_TIMEOUT )
        {
            CloseHandle( g_hSem );
            TRACE( "No more instances allowed" );
            return false;
        }
    }

    RECT posRect;
    posRect.left = left;
    posRect.top = top;
    posRect.right= posRect.left + width;
    posRect.bottom = posRect.top + height;

    g_pListBox = new SelectorList( hwndParent,
                                   szTitle,
                                   configFlags,
                                   positionFlags,
                                   posRect );

    return true;
    TRACE( "Done config" );
}

bool __export __stdcall Select_Init()
{
    if( 0 == g_pListBox )
    {
        // ensure that list box has been created
        return false;
    }



    HANDLE hEvent = CreateEvent( 0, true, false, 0 );
    if( 0 == hEvent )
    {
        TRACE( "Error from CreateEvent: " << GetLastError() );
        return false;
    }

    _beginthreadNT( GuiThread, 4096, hEvent, 0, 0, &g_dwThreadId );

    // wait for message loop to start before returning control to TSE
    WaitForSingleObject( hEvent, INFINITE );
    return true;
}

// returns 0 on success, -1 on failure, 1 if file already exists
int __export __stdcall Select_AddFile( const char* fileName,
                                       int bufferId,
                                       bool fFileChanged )
{
    // require valid filename (at least a single char)
    if( lstrlen( fileName ) <= 0 )
    {
        TRACE( "Invalid length of filename" );
        return -1;
    }

    TRACE( "Adding file: " << fileName << " " << bufferId );
    if( g_setFileInfo.count( FileInfo( fileName, bufferId, fFileChanged )))
    {
        return 1;
    }

    pair< FileInfoSet::iterator, bool > result = g_setFileInfo.insert(
                                                    FileInfo( fileName,
                                                              bufferId,
                                                              fFileChanged ) );
    if( result.second == false )
    {
        TRACE( "Error on insert of " << fileName );
        return -1;
    }



    return 0;
}

int __export __stdcall Select_RemoveFile( const char* fileName, int bufferId )
{
    TRACE( "Removing " << fileName << " " << bufferId );
    return g_setFileInfo.erase( FileInfo( fileName, bufferId, false ));
}

void __export __stdcall Select_Redraw()
{
    g_pListBox->SetRedraw( false );
    g_pListBox->ClearContent();

    // rebuild vector to ensure that it is always in sync with map
    g_vecFileInfoPtr.clear();
    FileInfoSet::iterator iter = g_setFileInfo.begin();

    while( iter != g_setFileInfo.end() )
    {
        g_vecFileInfoPtr.insert( g_vecFileInfoPtr.end(), (FileInfo*)&(*iter) );
        g_pListBox->InsertString( (*iter).FileName() );
        iter++;
    }

    if( g_pListBox->ShowNumFiles() )
    {
        strstream strm;
        string desc( "files" );
        if( g_setFileInfo.size() == 1 )
            desc = "file";
        if( g_setFileInfo.size() <= 0 )
            desc = "empty";

        strm << g_pListBox->Title() << " (" << g_setFileInfo.size() << " " << desc << ")" << ends;
        g_pListBox->SetTitle( strm.str() );
    }

    // changed - invalidate previous indexes
    g_pListBox->ResetChoice();

    g_pListBox->SetRedraw( true );
}

void __export __stdcall Select_HiliteCurrentFile( const char* fileName )
{
    g_pListBox->SelectString( fileName );
}

// return TSE buffer id of user choice
int __export __stdcall Select_OnIdle()
{
    if( g_pListBox->ParentMoved() )
    {
        g_pListBox->OnParentMove();
    }

    int idx;
    bool forRemoval;

    if( !g_pListBox->Choice( idx, forRemoval, true ) )
    {
        return 0;
    }

    TRACE( "idx: " << idx );
    int bufId = g_vecFileInfoPtr[ idx ]->BufferId();

    if( forRemoval )
    {
        bufId *= -1;
    }

    TRACE( "Buffer id: " << bufId );
    return bufId;
}

int __export __stdcall Select_ExitTSE()
{
    if( g_pListBox->ExitTSE() )
    {
        g_pListBox->ExitTSE( false );
        return true;
    }
    return false;
}
